/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-30 17:09:45 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "DatabaseManager.h"

static NSString* CookieDBStorageDirectory = nil;
@interface DatabaseManager(Private)
-(NSString *)_getCookieDBName;
@end

@implementation DatabaseManager(Private)
-(NSString *)_getCookieDBName
{
  if (CookieDBStorageDirectory == nil)
    {
      NSFileManager* manager;
      BOOL isDir, exists;
      NSString *storagePath;

      storagePath = [NSSearchPathForDirectoriesInDomains
                (NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
      storagePath = [storagePath stringByAppendingPathComponent:
                [[NSProcessInfo processInfo] processName]];
      storagePath = [NSString stringWithFormat:@"%@", storagePath];
      ASSIGN(CookieDBStorageDirectory, storagePath);

      manager = [NSFileManager defaultManager];
      exists = [manager fileExistsAtPath: CookieDBStorageDirectory
                isDirectory: &isDir];
      if (exists)
        {
          if (isDir == NO)
            {
              [[NSException exceptionWithName:
                        @"CookieDBStorageDirectoryIsNotADirectory"
                                       reason:
                        @"The storage directory for cookie DB "
			@"storage is not a directory."
                                     userInfo: nil] raise];
            }
        }
      else
        {
          NSError *error;
          if ([manager createDirectoryAtPath: CookieDBStorageDirectory
                 withIntermediateDirectories:YES
                                  attributes: nil
                                       error:&error] == NO)
            {
              [[NSException exceptionWithName:
                        @"CookieDBStorageDirectoryCreationFailed"
                                       reason:
                        @"Creation of the storage directory for "
			@"the cookie database failed."
                                     userInfo: nil] raise];
            }
        }
    }
  return [CookieDBStorageDirectory stringByAppendingPathComponent:
						@"CookieDB.sqlite3"];
}
@end

@implementation DatabaseManager
+(id) sharedDatabaseManager
{
  static DatabaseManager *_sharedDatabaseManager = nil;

  if (! _sharedDatabaseManager)
    {
      _sharedDatabaseManager = [[DatabaseManager allocWithZone:
						[self zone]] init];
    }
  return _sharedDatabaseManager;
}

-(id)init
{
  NSUserDefaults *defs;
  NSMutableArray *result;

  sortOrder = @"ASC";

  defs = [NSUserDefaults standardUserDefaults];
  [defs registerDefaults:
    [NSDictionary dictionaryWithObjectsAndKeys:
      [NSDictionary dictionaryWithObjectsAndKeys:
        [NSDictionary dictionaryWithObjectsAndKeys:
          [self _getCookieDBName], @"Database",
          @"", @"User",
          @"", @"Password",
          @"SQLite", @"ServerType",
          nil],
        @"CookieDB",
        nil],
      @"SQLClientReferences",
      nil]
    ];

  cookieDB = [[SQLClient clientWithConfiguration: nil 
					    name: @"CookieDB"] retain];
  [cookieDB execute: @"CREATE TABLE IF NOT EXISTS cookies ( "
                @"clientHost TEXT, serverHost TEXT, "
                @"baseDomain TEXT, name TEXT, value TEXT, "
		@"host TEXT, path TEXT, userAgent TEXT, "
                @"CONSTRAINT uniqueid UNIQUE "
		@"(clientHost,serverHost,name, host, value) "
		@"PRIMARY KEY (clientHost,serverHost,baseDomain,name))",
                nil];
  [cookieDB execute: @"CREATE INDEX IF NOT EXISTS basedomain ON cookies \
		(baseDomain, clientHost, serverHost)", nil];
  [cookieDB execute: @"CREATE TABLE IF NOT EXISTS version ( "
		@"version REAL )", nil];
  result = [cookieDB  query: @"SELECT version FROM version", nil];
  if (result != nil)
    {
      if ([result count] == 0)
	{
          // table was just created above
	  // with version 0.3, a timestamp field is added to the cookies table
          [cookieDB execute: @"INSERT INTO version(version)"
				@"values(0.3)", nil];
	  [cookieDB execute: @"ALTER TABLE cookies "
				@"ADD COLUMN timestamp TEXT", nil];
	  [cookieDB execute: @"UPDATE cookies SET timestamp=datetime() "
				@"WHERE timestamp IS NULL", nil];
	}
    }
  
  return self;
}

- (void) setSortRow: (NSString *) _sortrow
{
  ASSIGN(sortRow, _sortrow);
}
- (void) setSortOrder: (NSString *) _sortorder
{
  ASSIGN(sortOrder, _sortorder);
}

-(void)saveCookiesToDB:(Payload *)_payload
{
  NSString *query;
  for (id cookieName in [[_payload cookies] allKeys])
    {
      NSString *cookieValue = [[_payload cookies] objectForKey: cookieName];
      query = [NSString stringWithFormat:@"INSERT OR REPLACE INTO \
                cookies(clientHost,serverHost,baseDomain,host,name,\
			value,path,userAgent,timestamp) \
                values(%@, %@, %@, %@, %@, %@, %@, %@, datetime())",
                        [cookieDB quoteString:[_payload clientIP]], 
			[cookieDB quoteString:[_payload serverIP]],
                        [cookieDB quoteString:[_payload domain]], 
			[cookieDB quoteString:[_payload host]], 
			[cookieDB quoteString:cookieName],
                        [cookieDB quoteString:cookieValue], 
			[cookieDB quoteString:[_payload urlPath]],
			[cookieDB quoteString:[_payload userAgent]]];
      [cookieDB execute: query, nil];
      NSLog(@"saved cookie: %@ for %@ -> %@ (%@)", 
	cookieName, [_payload clientIP], [_payload serverIP], [_payload host]);
    }
}

- (NSArray *) getClientIPs
{
  NSString *query;
  NSMutableArray *records;

  if (sortRow == nil)
    {
      sortRow = @"clientHost";
    }
  query = [NSString stringWithFormat:@"SELECT "
		@"clientHost,count(DISTINCT host) AS count, "
		@"max(timestamp) AS timestamp "
		@"FROM cookies GROUP BY clientHost ORDER BY %@ %@",
		sortRow, sortOrder];
  records = [cookieDB query: query, nil];
  sortRow = nil;
  if ([sortOrder isEqual:@"DESC"])
    {
      sortOrder = @"ASC";
    }
  else
    {
      sortOrder = @"DESC";
    }
  return records;
}

- (NSArray *) getHostnames
{
  NSString *query;
  NSMutableArray *records;

  if (sortRow == nil)
    {
      sortRow = @"host";
    }
  query = [NSString stringWithFormat:@"SELECT host,baseDomain,"
		@"count(DISTINCT clientHost) AS count,"
		@"max(timestamp) AS timestamp FROM cookies GROUP BY host "
		@"ORDER BY %@ %@", sortRow, sortOrder];
  records = [cookieDB query: query, nil];
  sortRow = nil;
  if ([sortOrder isEqual:@"DESC"])
    {
      sortOrder = @"ASC";
    }
  else
    {
      sortOrder = @"DESC";
    }
  return records;
}

- (NSArray *) getHostsAccessedbyClient: (NSString *)_clientIP
{
  NSString *query;
  NSMutableArray *records;

  if (sortRow == nil)
    {
      sortRow = @"host";
    }
  query = [NSString stringWithFormat:@"SELECT DISTINCT "
		@"host,baseDomain,max(timestamp) AS timestamp from cookies "
		@"WHERE clientHost=%@ GROUP BY host ORDER BY %@ %@",
		[cookieDB quoteString:_clientIP], sortRow, sortOrder];
  records = [cookieDB query: query, nil];
  sortRow = nil;
  if ([sortOrder isEqual:@"DESC"])
    {
      sortOrder = @"ASC";
    }
  else
    {
      sortOrder = @"DESC";
    }
  return records;
}

- (NSArray *) getClientsAccessingHost: (NSString *)_targetHost
{
  NSString *query;
  NSMutableArray *records;

  if (sortRow == nil)
    {
      sortRow = @"clientHost";
    }
  query = [NSString stringWithFormat:@"SELECT DISTINCT "
		@"clientHost,max(timestamp) from cookies "
		@"WHERE host=%@ GROUP BY clientHost ORDER BY %@ %@",
		[cookieDB quoteString:_targetHost], sortRow, sortOrder];
  records = [cookieDB query: query, nil];
  sortRow = nil;
  if ([sortOrder isEqual:@"DESC"])
    {
      sortOrder = @"ASC";
    }
  else
    {
      sortOrder = @"DESC";
    }
  return records;
}

-(NSArray *) getCookiesForClientIP:(NSString *)_clientIP 
		     andTargetHost:(NSString *)_targetHost 
{
  NSString *query;
  NSMutableArray *records;

  query = [NSString stringWithFormat:@"SELECT \
	name,value,baseDomain,path,userAgent,host FROM cookies \
	WHERE host=%@ and clientHost=%@ ORDER BY baseDomain", 
		[cookieDB quoteString:_targetHost],
		[cookieDB quoteString:_clientIP]];
  records = [cookieDB query: query, nil];
  return records;
}

-(NSArray *) getDistinctURLsForClientIP:(NSString *)_clientIP 
		     andTargetHost:(NSString *)_targetHost 
{
  NSString *query;
  NSMutableArray *records;

  query = [NSString stringWithFormat:@"SELECT DISTINCT \
	path FROM cookies \
	WHERE host=%@ and clientHost=%@",
	[cookieDB quoteString:_targetHost],
	[cookieDB quoteString:_clientIP]];
  records = [cookieDB query: query, nil];
  return records;
}

-(void) deleteCookiesForClient:(NSString *)_client
{
  NSString *query;

  query = [NSString stringWithFormat:@"DELETE FROM cookies "
		@"WHERE clientHost=%@",
		[cookieDB quoteString:_client]];
  [cookieDB execute:query, nil];
}
-(void) deleteCookiesForClient:(NSString *)_client
		       andHost:(NSString *)_host
{
  NSString *query;

  query = [NSString stringWithFormat:@"DELETE FROM cookies "
		@"WHERE clientHost=%@ AND host=%@",
		[cookieDB quoteString:_client],
		[cookieDB quoteString:_host]];
  [cookieDB execute:query, nil];
}
-(void) deleteCookiesForHost: (NSString *)_host
{
  NSString *query;

  query = [NSString stringWithFormat:@"DELETE FROM cookies "
		@"WHERE host=%@",
		[cookieDB quoteString:_host]];
  [cookieDB execute:query, nil];
}

@end
