/*
   Project: ImpersonatorToolKit

   Copyright (C) 2013 Sebastian Reitenbach

   Author: Sebastian Reitenbach

   Created: 2013-06-25 19:11:32 +0200 by sebastia

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "LinkLayer.h"
#include <sys/types.h>
#include <netinet/in_systm.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip6.h>

#define ETH_P_IP  0x0800
#define ETH_P_IP6 0x86DD

struct ethhdr {
  unsigned char dest[ETH_ADDRLEN];
  unsigned char src[ETH_ADDRLEN];
  u_int16_t     proto;
} __attribute__((packed));

#define LLAYER_PROTO_OFFSET	12
#define IP_HEADER_OFFSET	14
#define TCP_PROTO_OFFSET	14
#define TCP_DATA_OFFSET		24

@implementation LinkLayer

-(void)dealloc
{
  [srcIP release];
  [dstIP release];
  [super dealloc];
}

-(NSUInteger) offsetNext
{
  return offsetNext;
}
-(NSUInteger) offsetData
{
  return offsetData;
}
-(NSInteger) nextProto
{
  return nextProto;
}
- (NSUInteger) srcPort
{
  return srcPort;
}
- (NSUInteger) dstPort
{
  return dstPort;
}

- (NSString *) srcIP
{
  return srcIP;
}
- (NSString *) dstIP
{
  return dstIP;
}
- (NSUInteger) tcpSequence
{
  return tcpSequence;
}
- (NSUInteger) tcpFlags
{
  return tcpFlags;
}

- (BOOL) handleLinkLayerForDataLink: (DataLink *)_dataLink
                             packet: (NSData *)_packet
{
  unsigned short llnextproto = 0;
  struct ethhdr *eptr;
  switch ([_dataLink type])
    {
      case DLT_EN10MB:
        eptr = (struct ethhdr *) [_packet bytes];
        offsetNext = sizeof(struct ethhdr);
        llnextproto = ntohs(eptr->proto);
        switch (llnextproto)
          {
            case ETH_P_IP:
              nextProto = IPPROTO_IP;
              break;
            case ETH_P_IP6:
              nextProto = IPPROTO_IPV6;
              break;
            default:
              NSLog(@"link level next protocol (%hd) is not supported", llnextproto);
              return NO;  
          }
        break;
        
      default:
        NSLog(@"link-level (%@) header is not supported", [_dataLink name]);
        return NO;
    }
  return YES;  
}

- (BOOL) findTcpForPacket:(NSData *)_packet
{
  while (1)
    {
      switch(nextProto)
        {
	  case IPPROTO_TCP:		// we found what we are looking for
	    {
#warning it would crash randomly using commented out variant
	      //struct tcphdr *tcp = calloc(sizeof(struct tcphdr +1), 1);
	      struct tcphdr *tcp = calloc(sizeof(struct sockaddr_storage), 1);
              NSRange range;
	      range.location = offsetNext;
	      range.length = sizeof(struct tcphdr);
	      [_packet getBytes:(struct tcphdr *)tcp range:range];
	      srcPort = tcp->th_sport;
	      dstPort = tcp->th_dport;
	      offsetNext += tcp->th_off << 2;	// update the offset
	      tcpSequence = ntohl(tcp->th_seq);
	      tcpFlags = tcp->th_flags;
	      offsetData = offsetNext;
	      free((struct sockaddr_storage *)tcp);
	      return YES;
	    }
	  case IPPROTO_IPIP:	// IPIP tunnel
	  case IPPROTO_IP:	// IP packet
	    {
#warning it would crash randomly using commented out variant
	      //struct ip *ip = calloc(sizeof(struct ip), 1);
	      struct ip *ip = calloc(sizeof(struct sockaddr_storage), 1);
	      struct sockaddr_in *ps = calloc(sizeof(struct sockaddr_in), 1);
   	      struct sockaddr_in *pd = calloc(sizeof(struct sockaddr_in), 1);
	      NSRange range;
	      range.location = IP_HEADER_OFFSET;
	      range.length = IP_HEADER_OFFSET + sizeof(struct ip);
	      [_packet getBytes:(struct ip *)ip range:range];
	      ps->sin_family = AF_INET;
	      memcpy(&ps->sin_addr.s_addr, &ip->ip_src, sizeof(struct in_addr));
	      srcPort = ps->sin_port;
	      srcIP = [NSString stringWithFormat:@"%s", inet_ntoa(ip->ip_src)];
	      pd->sin_family = AF_INET;
	      memcpy(&pd->sin_addr.s_addr, &ip->ip_dst, sizeof(struct in_addr));
	      dstIP = [NSString stringWithFormat:@"%s", inet_ntoa(ip->ip_dst)];
	      dstPort = pd->sin_port;
	      nextProto = ip->ip_p;
	      offsetNext += ip->ip_hl << 2;	// push the offset to the next layer
	      free((struct sockaddr_storage *)ip);
	      free(ps);
	      free(pd);
	      break;
	    }
	  case IPPROTO_IPV6:      // IPv6 packet
	    {
	      struct ip6_hdr *ip6hdr = calloc(sizeof(struct ip6_hdr), 1);
	      struct sockaddr_in6 *ps = calloc(sizeof(struct sockaddr_in6), 1);
	      struct sockaddr_in6 *pd = calloc(sizeof(struct sockaddr_in6), 1);
	      char ip6str[INET6_ADDRSTRLEN];
	      NSRange range;
	      range.location = IP_HEADER_OFFSET;
	      range.length = IP_HEADER_OFFSET + sizeof(struct ip6_hdr);
	      [_packet getBytes:(struct ip6_hdr *)ip6hdr range:range];
	      ps->sin6_family = AF_INET6;
	      memcpy(&ps->sin6_addr.__u6_addr, &ip6hdr->ip6_src, sizeof(struct in6_addr));
	      srcPort = ps->sin6_port;
	      inet_ntop(AF_INET6, &(ps->sin6_addr), ip6str, INET6_ADDRSTRLEN);
	      srcIP = [NSString stringWithFormat:@"%s", ip6str];      
	      pd->sin6_family = AF_INET6;
	      memcpy(&pd->sin6_addr.__u6_addr, &ip6hdr->ip6_dst, sizeof(struct in6_addr));
	      dstPort = pd->sin6_port;
	      inet_ntop(AF_INET6, &(pd->sin6_addr), ip6str, INET6_ADDRSTRLEN);
	      dstIP = [NSString stringWithFormat:@"%s", ip6str];      
	      nextProto = ip6hdr->ip6_nxt;
	      offsetNext += sizeof(struct ip6_hdr);
	      free(ip6hdr);
	      free(ps);
	      free(pd);
	      break;
	    }
          case IPPROTO_DSTOPTS:   /* destination option */
          case IPPROTO_ROUTING:   /* routing header */
            {
              struct ip6_ext * ip6ext = calloc(sizeof(struct ip6_ext), 1);
	      NSRange range;
	      range.location = offsetNext;
	      range.length = offsetNext + sizeof(struct ip6_ext);

	      [_packet getBytes:(struct ip6_ext *)ip6ext range:range];

              /* update nextproto and offset */
              nextProto = ip6ext->ip6e_nxt;
              offsetNext += ip6ext->ip6e_len << 3;
	      free(ip6ext);
              break;
           }
		
	  default:
		goto end;	// unsupported protocol
		
        }
    }

end:
	NSLog(@"unsupported protocol dataframe (%"PRIiPTR")", nextProto);
	return NO;
}

@end
