/*
   Project: ImpersonatorToolKit
   Copyright: Sebastian Reitenbach
   Author: Sebastian Reitenbach

   Created: 2013-06-23 12:45:48 +0200 by sebastia

   License: GPLv2 or later
*/

#import <Foundation/Foundation.h>
#import <SnifferController.h>
#import <WebServerHandler.h>

void usage()
{
  printf("ImpersonatorToolKit options\n");
  printf("-help YES\t\t\tthis help page\n");
  printf("-ProxyOnly YES\t\t\tno sniffing, only run proxy, default: NO\n");
  printf("-ProxyPort <Nr.>\t\tproxy listen on port <Nr.>, default: 8080\n");
  printf("-inputFile <filename>\t\tno proxy, just fill database from pcap dump\n");
  printf("-device <device>\t\tsniff on the given device\n");
  printf("-promiscuous <YES/NO>\t\tsniff in promiscuous mode, default: NO\n");
  printf("-readTimeout <seconds>\t\tsniffing read timeoout, default: 0\n");
  printf("-bpfFilter <filter expression>\tBPF filter, default: tcp and port 80\n");
  printf("-optimizeFilter <YES/NO>\tBPF filter optimization, default: NO\n");
  printf("-listDataLinkTypesForDevice <device>\tlist datalink types for device\n");

  printf("\nIf no input file nor a device given, it will try to "
	 "find a device automatically and start sniffing and the proxy.\n");
  printf("\nDebugging options\n");
  printf("-ConnectionManagerDebug <YES/NO>\tdebug connection manager, default: NO\n");
  printf("-SnifferControllerDebug <YES/NO>\tdebug sniffer controller, default: NO\n");
  printf("-WebServerHandlerDebug <YES/NO>\t\tdebug webserver handler, default: NO\n");

}

int
main(int argc, const char *argv[])
{
  id pool = [[NSAutoreleasePool alloc] init];
  NSString *device;
  NSString *inputFile;
  BOOL promiscuous;
  NSInteger readTimeout;
  NSString *bpfFilter;
  BOOL optimizeFilter;
  WebServerHandler *wsHandler;
  NSString *proxyPort;
  SnifferController *sniffer;
  BOOL proxyOnly;

  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];

  if ([ud boolForKey:@"help"])
    {
      usage();
      return 0;
    }
  NSLog(@"for help run %s -help YES", argv[0]);

  device = [ud stringForKey:@"listDataLinkTypesForDevice"];
  if (device != NULL && ![device isEqual:@""])
    {
      int cnt;		// number of link types found
      int i;		// loop counter
      int *dlt_buf;	// pointer to buffer holding found link types
      sniffer = [[SnifferController alloc] init];
      [sniffer setDevice:device];
      [sniffer createSniffingHandle];
      [sniffer activateSniffingHandle];

      cnt = [sniffer getDatalinksIntoArray: &dlt_buf];
      if (cnt == -1)
	{
	  NSLog(@"found no datalinktypes for device: %@", device);
          RELEASE(sniffer);
	  return -1;
	}
      else
	{

          NSLog(@"found %i datalinktypes for device: %@", cnt, device);
          for (i = 0;i<cnt;i++)
            {
              NSLog(@"%@, %@",
                [PcapKit getDataLinkNameForType:dlt_buf[i]],
                [PcapKit getDataLinkDescriptionForType:dlt_buf[i]]);
            }
	  return 0;
	}
    }

  proxyOnly = [ud boolForKey:@"ProxyOnly"];
  // configure the server from defaults
  proxyPort = [ud stringForKey: @"ProxyPort"];
  if (proxyPort == nil)
    proxyPort = @"8080";
  NSLog(@"proxy listening on port: %@", proxyPort);

  // configure the sniffing from defaults
  inputFile = [ud stringForKey:@"inputFile"];
  device = [ud stringForKey:@"device"];
  if (device != NULL && inputFile != NULL)
    {
      NSLog(@"device and input file given, choose only one of those");
      return 1;
    }

  wsHandler = [WebServerHandler new];
  [wsHandler setWebServerPort: proxyPort];
  if (!proxyOnly)
  [NSThread detachNewThreadSelector:@selector(runWebserver)
			   toTarget: wsHandler
			 withObject: nil];
  else
    [wsHandler runWebserver];
  
  promiscuous = [ud boolForKey:@"promiscuous"];
  readTimeout = [ud integerForKey:@"readTimeout"];
  if ((bpfFilter = [ud stringForKey:@"bpfFilter"]) == NULL)
    bpfFilter = @"tcp and port 80";
  optimizeFilter = [ud boolForKey:@"optimizeFilter"];
    
  if (!proxyOnly)
    {
      sniffer = [[SnifferController alloc] init];
      if (inputFile != NULL)
        {
          [sniffer setInputFile:inputFile];
        }
      else
        {
          if (device == NULL)
	    {
	      // try to automatically find a device
	      if (![sniffer findDeviceAutomatically])
	        {
	          NSLog(@"no input file or input device given, "
			@"and also unable to find device by myself");
		  RELEASE(sniffer);
	          return 2;
	        }
	    }
          else
	    {
	      [sniffer setDevice:device];
	    }
        }

      [sniffer setReadTimeout:readTimeout];
      [sniffer setBpfFilter:bpfFilter];
      [sniffer setOptimizeFilter:optimizeFilter];
  
      if ([sniffer inputFile])
        [sniffer openOfflineStream]; 
      else
        {
          [sniffer setPromiscuous:promiscuous];
          [sniffer openLiveStream]; 
        }

      [sniffer applyFilter];
      [sniffer setDelegate];
      [sniffer startPcapLoop];
  
      RELEASE(sniffer);
    }
  RELEASE(wsHandler);
  RELEASE(pool);

  return 0;
}

